/****************************************************************************
; * 																		
; * 	File		:	app.cpp
; * 																		
; * 	Description :	Direct X Wrapper Tool
; * 					
; * 	Copyright (C) 1993-2001 Realtech									
; * 																		
; * 	Permission to use, copy, modify, distribute and sell this software	
; * 	and its documentation for any purpose is hereby granted without fee,
; * 	provided that the above copyright notice appear in all copies and	
; * 	that both that copyright notice and this permission notice appear	
; * 	in supporting documentation.  Realtech makes no representations 	
; * 	about the suitability of this software for any purpose. 			
; * 	It is provided "as is" without express or implied warranty. 		
; * 																		
; ***************************************************************************/
#include <stdio.h>
#include <windows.h>
#include "win32/w32_rfc.h"
#include "win32/w32_reg.h"
#include "win32/w32_dx.h"
#include "resource.h"

#define APP_REGPATH "Software\\Realtech.VR"

W32RFC *rfc;
W32REG	reg;

class MainDlg : public W32RFC_Dlg {
	public:
		MainDlg();
		~MainDlg();
		void OnInitDialog();
		void OnCommand(unsigned dlg);
		void EnableWrapper(int mode);
		void DisableWrapper(int mode);
		void Uninstall();
		int copyfromsystemtoapp(const char *s1, const char *s2);
		int copyfromapptosystem(const char *s1, const char *s2);

};

MainDlg::MainDlg() : W32RFC_Dlg(rfc, IDD_PANEL)
{
}

MainDlg::~MainDlg()
{
}

long file_size(FILE *fp)
{
	long curpos, length;
	curpos = ftell(fp);
	fseek(fp, 0L, SEEK_END);
	length = ftell(fp);
	fseek(fp, curpos, SEEK_SET);
	return length;
}


void file_copy(FILE *pfDest, FILE *pfSrc, unsigned long lLength)
{
	const unsigned int COPY_BLOCK_SIZE	= 65536;
	char *temp;
	temp =(char *) malloc(COPY_BLOCK_SIZE*sizeof(char));
	while (!(lLength<COPY_BLOCK_SIZE))
	{
		fread(temp,sizeof(char),COPY_BLOCK_SIZE,pfSrc);
		fwrite(temp,sizeof(char),COPY_BLOCK_SIZE,pfDest);
		lLength-=COPY_BLOCK_SIZE;
	}
	if (lLength>0)
	{
		fread(temp,sizeof(char),lLength,pfSrc);
		fwrite(temp,sizeof(char),lLength,pfDest);
	}
	free(temp);
	return;
}

int filecopy(const char *szDest, const char *szSrc)
{
/*
	char tmp[128];
	sprintf(tmp,"copy %s to %s", szSrc, szDest);
	rfc->InfoBox(tmp);
*/

	FILE *pfSrc = fopen(szSrc, "rb");
	if (!pfSrc)
	{		
		return -1;
	}

	FILE *pfDest = fopen(szDest, "wb");
	if (!pfDest)
	{
		fclose(pfSrc);
		return -1;
	}
	file_copy(pfDest, pfSrc, file_size(pfSrc));
	return 0;
}

int MainDlg::copyfromsystemtoapp(const char *s1, const char *s2)
{
	char f1[_MAX_PATH];
	
	GetSystemDirectory(f1, _MAX_PATH);
	strcat(f1,"\\");
	strcat(f1, s1);

	return filecopy(s2, f1);
}

int MainDlg::copyfromapptosystem(const char *s1, const char *s2)
{
	char f2[_MAX_PATH];

	GetSystemDirectory(f2, _MAX_PATH);
	strcat(f2,"\\");
	strcat(f2, s2);

	return filecopy(f2, s1);
}

void MainDlg::EnableWrapper(int mode)
{
	if (mode & 1)
	{
		if (!reg.GetKey("GLWrapper", "d3d8"))
		{
			int b = copyfromsystemtoapp("d3d8.dll","d3d8.bak");
			int a = copyfromapptosystem("d3d8.dll","d3d8.dll");

			if ((b) || (a))
			{
				if (a)
				{
					_ctx->ErrorBox("System Patch Error!\n\nCan't copy d3d8.dll (Direct3D 8 Immediate Mode)\nfrom system to d3d8.bak\n");
				}
				if (b)
				{
					_ctx->ErrorBox("System Patch Error!\n\nCan't copy wrapper d3d8.dll into system directory\n");
				}
				reg.SetKey("GLWrapper", "d3d8",1);
			}
		}			
	}

	if (mode & 2)
	{
		if (!reg.GetKey("GLWrapper", "d3dx8"))
		{
			int a = copyfromsystemtoapp("d3dx8.dll","d3dx8.bak");
			int b = copyfromapptosystem("d3dx8.dll","d3dx8.dll");

			if ((b) || (a))
			{
				if (a)
				{
					_ctx->ErrorBox("System Patch Error!\n\nCan't copy d3dx8.dll (Direct3D 8 Retained Mode)\nfrom system to d3d8.bak");
				}
				if (b)
				{
					_ctx->ErrorBox("System Patch Error!\n\nCan't copy wrapper d3dx8.dll into system directory.");
				}
				reg.SetKey("GLWrapper", "d3dx8",1);
			}
			
		}
	}
}

void MainDlg::Uninstall()
{
	DisableWrapper(1);
	DisableWrapper(2);
	ShellExecute(0,"open","uninst.exe",NULL, NULL, SW_NORMAL);
	Close(1);
}

void MainDlg::DisableWrapper(int mode)
{
	if (mode & 1)
	{
		if (reg.GetKey("GLWrapper", "d3d8"))
		{
			if (!copyfromapptosystem("d3d8.bak","d3d8.dll"))
			reg.SetKey("GLWrapper", "d3d8",0);
			else
				_ctx->ErrorBox("Can't disable d3d8");
		}
	}

	if (mode & 2)
	{
		if (reg.GetKey("GLWrapper", "d3dx8"))
		{
			if (!copyfromapptosystem("d3dx8.bak","d3dx8.dll"))
			reg.SetKey("GLWrapper", "d3dx8",0);
			else
				_ctx->ErrorBox("Can't disable d3dx8");
		}
	}
}


void MainDlg::OnInitDialog()
{
	DWORD dwDXVersion, dwDXPlatform;

	reg.SetKey("GLWrapper", "boot", 1);
	DX_GetVersion(&dwDXVersion, &dwDXPlatform);
	char tmp[128];

	reg.SetKey("GLWrapper", "boot", 2);
	sprintf(tmp,"Direct X %d.%d installed.", dwDXVersion>>8, dwDXVersion&0xff);
	_ctx->SetText(IDC_INFO, tmp);
	_ctx->SetIcon(IDI_ICON1 );

	_ctx->SetCheck(IDC_EMU, reg.GetKey("GLWrapper", "d3d8"));
	_ctx->SetCheck(IDC_EMU2, reg.GetKey("GLWrapper", "d3dx8"));
	reg.SetKey("GLWrapper", "boot", 3);
}

void MainDlg::OnCommand(unsigned dlg)
{
	switch(dlg) {
		case IDC_APPLY:
		if (_ctx->GetCheck(IDC_EMU))
		{
			EnableWrapper(1);
		}
		else
			DisableWrapper(1);

		if (_ctx->GetCheck(IDC_EMU2))
		{
			EnableWrapper(2);
		}
		else
			DisableWrapper(2);
		break;

		case IDC_DEFAUT:
		break;

		case IDC_APPLY2:
			Uninstall();
		break;

	}
}




int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{	  
	rfc = new W32RFC(hInstance);
	reg.OpenPath(APP_REGPATH, HKEY_CURRENT_USER);
	int boot = reg.GetKey("GLWrapper", "boot");
	if (boot)
	{
		char tmp[256];
		sprintf(tmp,"Warning!\n\nThe wrapper didn't close properly last time at stage %d.\nIf it still fails, please contact support@realtech-vr.com.", boot);
		rfc->WarningBox(tmp, "Safe mode");
	}
	W32RFC_Dlg *pDialog = new MainDlg();
	pDialog->Go();
	delete pDialog;
	delete rfc;
	reg.SetKey("GLWrapper", "boot", 0);
	return 0;
}