/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Be Corp
 *
 *  File:       entrypoint.cpp
 *  Content:    GL initialization code
 *
 ***************************************************************************/

#include <DirectGLWindow.h>
#include "../gl_init.h"

#define MAX_DEVICE_NAME_LENGTH 128


typedef struct _devicemode {
	uint32 width;
	uint32 height; 
	uint32 color; 
	uint32 depth; 
	uint32 stencil;
	uint32 accum;
}GLDISPLAYMODE;

typedef struct _devicetype {
	uint32	device_id;
	uint32	monitor;
	char	name [ MAX_DEVICE_NAME_LENGTH ];
}GLDEVICE;
	
// BeOS GL Driver
struct gl_driver_information {
	BDirectGLWindow	*	hwnd;
	int					nmode;
	int					ndevice;
	
	GLDISPLAYMODE	*	dispmode;
	GLDEVICE		*	devices;

	uint32				deviceId;	

};

class D3D8_GLWindow : public BDirectGLWindow {
public:
	D3D8_DGLWindow(BRect rect, const char *title, struct gl_driver_information *pDevice, thread_id iThreadID);
	status_t RestartGL(, struct gl_driver_information *pDevice);
	void MessageReceived( BMessage *msg );
	void DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum );
	void VideoModeInfo( uint32 width, uint32 height, uint32 color, uint32 depth, uint32 stencil, uint32 accum );
	void SetFullScreenMode(bool bFs, int lx, int ly);
	static void render_thread(void *data);

	struct gl_driver_information	*	m_pDevice;
	bool								m_bInitialized;
	bool								m_bFullScreen;
	thread_id							m_tDrawThread;
};

// TODO: These buffer should allocate in AGP if possible (allocators used for vertex buffer ...)

void *glAllocateMemory(int size)
{
	return new BYTE[size];
}

void glFreeMemory(void *ptr)
{
	delete [] ptr;
}


// Get Display mode 
void GL_GetAdapterDisplayMode(struct gl_driver_information *pDevice, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{	
	return;
}
// Get adapter driver name
void GL_GetAdapterName(struct gl_driver_information *pDevice, int index, char *s, int numBytes)
{
	GLDEVICE	*	dev = pDevice->devices + index;
	strncpy(s, dev->name, numBytes);
	return;
}

// Get number of display modes available
int GL_GetAdapterCount(struct gl_driver_information *pDevice)
{
	return pDevice->ndevice;
}

// Get number of display modes available
int GL_GetAdapterModeCount(struct gl_driver_information *pDevice)
{
	return pDevice->nmode;
}
// Get adapter mode

void GL_EnumAdapterModes(struct gl_driver_information *pDevice, int index, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{
	GLDISPLAYMODE *pmode = pDevice->dispmode + index;
	*width = pmode->width;
	*height = pmode->height;
	*bpp = pmode->color;
	*refreshRate = 0;
}

// Page flipping
void GL_PresentDevice(struct gl_driver_information *pDevice)
{
	pDevice->hwnd->SwapBuffers();
	pDevice->hwnd->YieldCurrent();
	return;
}

// Page flipping
int GL_CreateDevice(struct gl_driver_information **pDevice, void *hwnd, int deviceId, int width, int height)
{
	gl_driver_information *p = new gl_driver_information;
	p->deviceId = deviceId;
	*pDevice = 0;
	p->hwnd = new D3D8_DGLWindow(BRect(0,0,width,height), "Direct3D8", p, hwnd);
	*pDevice = p;
	return 0;
}

void GL_ReleaseDevice(struct gl_driver_information *pDevice)
{
	pDevice->hwnd->Quit();
	return;
}

/////////////////////////

int32 D3D8_DGLWindow::render_thread( void *data )
{
	D3D8_DGLWindow *wnd = (D3D8_DGLWindow*)data;
	wnd->MakeCurrent();
	m_pFunc();
	wnd->ReleaseCurrent();
	return 0;
}

D3D8_DGLWindow::D3D8_DGLWindow(BRect rect, const char *title, struct gl_driver_information *pDevice, thread_id iThreadID) 
: BDirectGLWindow(	rect, 
					title,
					B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
					B_ASYNCHRONOUS_CONTROLS | B_NOT_ZOOMABLE ), m_pDevice(pDevice), m_bInitialized(false), m_bFullScreen(false)
{

	m_pDevice->nmodes = 0;
	m_pDevice->ndevices = 0;
	EnumerateDevices( BGL_MONITOR_ANY, BGL_ANY | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ); 

	// Start the drawing thread.
	m_tDrawThread = spawn_thread( render_thread, "D3D8 drawing thread", B_DISPLAY_PRIORITY, this );
	if( m_tDrawThread < 0 )
	{
		fprintf( stderr, "Can't spawn drawing thread, bailing out...\n" );
-		be_app->PostMessage( B_QUIT_REQUESTED );
		return B_ERROR;
	}


	if (RestartGL(pDevice->deviceId) < B_OK) 
		return;
}

status_t D3D8_DGLWindow::RestartGL()
{
	status_t ret;

	if (m_bInitialized)
	{
		ShutdownGL();
		m_bInitialized = false;
	}
	m_bInitialized = true;

	m_bFullScreen = false;
	if( InitializeGL( deviceID, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ) < B_OK )
	{
		fprintf( stderr, "Error from InitializeGL \n" );
		be_app->PostMessage( B_QUIT_REQUESTED );
	}
	return ret;
}

void D3D8_DGLWindow::MessageReceived( BMessage *msg )
{
	switch( msg->what )
	{
		return BDirectWindow::MessageReceived(msg); break;
	}
	return; 			
}

void D3D8_DGLWindow::DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum )
{
	GLDEVICE *dev = m_pDevice->devices + m_pDevice->ndevices;
	dev->device_id	= device_id;
	dev->monitor	= monitor;
	strncpy(dev->name, name, 128);
	m_pDevice->ndevices++;
	return;
}

void D3D8_DGLWindow::VideoModeInfo( uint32 width, uint32 height, uint32 color, uint32 depth, uint32 stencil, uint32 accum )
{
	GLDISPLAYMODE *dev = m_pDevice->dmodes + m_pDevice->nmodes;
	dev->width  = width;
	dev->height = height;
	dev->color  = color;
	dev->depth  = depth;
	dev->stencil = stencil;
	dev->accum = accum;
	m_pDevice->nmodes ++;
	return;
}
void D3D8_DGLWindow::SetFullScreenMode(bool bFs, int lx, int ly)
{
	if (!bFs)
	{
		FullscreenDisable();
		ResizeTo( lx - 1, ly - 1);
		m_bFullScreen = false;
	}
	else
	{
		FullscreenEnable( lx, ly);
		m_bFullScreen = true;
	}
}

