/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       ixfileformat.cpp - ANSI Code (requires STL support)
 *  Content:    .X file parser. based from http://www.microsoft.com/directx/dxm/help/d3drm/oview/xfileBinaryFormat.htm
 *
 ***************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "ixfileformat.h"

#define ASSERT assert

#define XOFFILE_FORMAT_MAGIC \
  ((long)'x' + ((long)'o' << 8) + ((long)'f' << 16) + ((long)' ' << 24))

#define XOFFILE_FORMAT_VERSION \
  ((long)'0' + ((long)'3' << 8) + ((long)'0' << 16) + ((long)'2' << 24))

#define XOFFILE_FORMAT_BINARY \
  ((long)'b' + ((long)'i' << 8) + ((long)'n' << 16) + ((long)' ' << 24))

#define XOFFILE_FORMAT_TEXT   \
  ((long)'t' + ((long)'x' << 8) + ((long)'t' << 16) + ((long)' ' << 24))

#define XOFFILE_FORMAT_COMPRESSED \
  ((long)'c' + ((long)'m' << 8) + ((long)'p' << 16) + ((long)' ' << 24))

#define XOFFILE_FORMAT_FLOAT_BITS_32 \
  ((long)'0' + ((long)'0' << 8) + ((long)'3' << 16) + ((long)'2' << 24))

#define XOFFILE_FORMAT_FLOAT_BITS_64 \
  ((long)'0' + ((long)'0' << 8) + ((long)'6' << 16) + ((long)'4' << 24))


//Record-bearing 

#define TOKEN_NAME 1
#define TOKEN_STRING 2
#define TOKEN_INTEGER 3
#define TOKEN_GUID 5
#define TOKEN_INTEGER_LIST 6
#define TOKEN_FLOAT_LIST 7

//Stand-alone 

#define TOKEN_OBRACE 10
#define TOKEN_CBRACE 11
#define TOKEN_OPAREN 12
#define TOKEN_CPAREN 13
#define TOKEN_OBRACKET 14
#define TOKEN_CBRACKET 15
#define TOKEN_OANGLE 16
#define TOKEN_CANGLE 17
#define TOKEN_DOT 18
#define TOKEN_COMMA 19
#define TOKEN_SEMICOLON 20
#define TOKEN_TEMPLATE 31
#define TOKEN_WORD 40
#define TOKEN_DWORD 41
#define TOKEN_FLOAT 42
#define TOKEN_DOUBLE 43
#define TOKEN_CHAR 44
#define TOKEN_BYTE 45
#define TOKEN_SWORD 46
#define TOKEN_SDWORD 47
#define TOKEN_VOID 48
#define TOKEN_LPSTR 49
#define TOKEN_UNICODE 50
#define TOKEN_CSTRING 51
#define TOKEN_ARRAY 52


IXFile::IXFile(FILE *fp) : m_fp(fp)
{
	memset(&m_Header, 0, sizeof(m_Header));
	if (fp)
		ParseFile();
}

IXFile::IXFile(const char *szFilename)
{
	FILE *fp = fopen(szFilename, "rb");
	memset(&m_Header, 0, sizeof(m_Header));
	m_fp = fp;
	if (fp)
		ParseFile();	
	fclose(fp);	
}
IXFile::~IXFile()
{
	
}

bool IXFile::ParseHeader()
{
	int nbRead = fread(&m_Header, sizeof(m_Header), 1, m_fp);
	#ifdef _LITTLE_ENDIAN_
			#error "This code is not yet modified for little endianness!!"
	#endif
	return (nbRead == 1);

}

bool IXFile::ParseFile()
{
	if (ParseHeader())
	{	
		if (*(long*)m_Header.m_szMagicNumber==XOFFILE_FORMAT_MAGIC)
		{
			//MSG("Found a valid .XOF file format\n");
			if (*(long*)m_Header.m_szFormatType==XOFFILE_FORMAT_TEXT)
			{
				//MSG("Text format (not supported!)\n");
				ParseTextFormat();
			}
			else
			if (*(long*)m_Header.m_szFormatType==XOFFILE_FORMAT_BINARY)
			{
				//MSG("Bin format\n");
				ParseBinFormat();
			}
		}
		else
			return false;
	}
	return false;
}


int IXFile::GetWord(char *tmp, int iBufSize)
{
	fgets(tmp,iBufSize,m_fp);
	return iBufSize;
}

bool IXFile::ParseTextFormat()
{
	char tmp[32];
	while (!feof(m_fp))
	{		
		GetWord(tmp, 32);
	}
	return true;
}

bool IXFile::ParseBinFormat()
{
	int inTemplate = 0;
	int inBrace = 0;
	int iWaitName = 0;
	char szTagName[128];

	IXFile_Mesh *pMesh = 0;
	int phase = 0;

	while (!feof(m_fp))
	{
		WORD idToken = 0;
		if (!fread(&idToken, sizeof(WORD), 1, m_fp))
			break;
		
		switch(idToken)
		{
			case TOKEN_TEMPLATE:
				inTemplate++;
				iWaitName = 1;
			break;

			case TOKEN_OBRACE:
			if (inBrace)
				inBrace++;
			iWaitName = 0;
			//MSG("{\n");
			break;

			case TOKEN_CBRACE:
			if (inBrace)
				inBrace--;

			if (inTemplate)
				inTemplate--;
			//MSG("}\n");
			break;

			case TOKEN_NAME:
			{				
				DWORD count;
				fread(&count, sizeof(DWORD), 1, m_fp);
				#ifdef _LITTLE_ENDIAN_
					#error "This code is not yet modified for little endianness!!"
				#endif
				fread(szTagName, sizeof(char), count, m_fp); szTagName[count] = 0;
				//MSG("Name=%s\n", szTagName);
				if (iWaitName)
				{
					iWaitName = 0;
					//MSG("Found %s\n", szTagName);
				}
				if (strcmp(szTagName, "Mesh")==0)
				{
					if (pMesh)
						m_pMeshes.push_back(pMesh);

					pMesh = new IXFile_Mesh;
					ASSERT(pMesh);					
				}
				phase = 0;
				
			}
			break;

			case TOKEN_STRING:
			{
				char szTokenName[128];
				DWORD count;
				fread(&count, sizeof(DWORD), 1, m_fp);
				#ifdef _LITTLE_ENDIAN_
					#error "This code is not yet modified for little endianness!!"
				#endif
				fread(szTokenName, sizeof(char), count, m_fp);	szTokenName[count] = 0;		 
				printf("String=%s\n", szTokenName);
			}
			break;

			case TOKEN_INTEGER:
			{				
				DWORD value;
				fread(&value, sizeof(DWORD), 1, m_fp);			
				//MSG("Int=%d\n", value);
			}
			break;

			case TOKEN_GUID:
			{
				char szGuid[16];
				DWORD count;
				fread(&count, sizeof(DWORD), 1, m_fp);
				#ifdef _LITTLE_ENDIAN_
					#error "This code is not yet modified for little endianness!!"
				#endif
				fread(&szGuid, 16, 1, m_fp);
				//MSG("Guid=%s\n", szGuid);
				
			}
			break;

			case TOKEN_INTEGER_LIST:
			{
				DWORD count;
				fread(&count, sizeof(DWORD), 1, m_fp);
				#ifdef _LITTLE_ENDIAN_
					#error "This code is not yet modified for little endianness!!"
				#endif
				DWORD *pTemp = new DWORD[count];
				ASSERT(pTemp);
				fread(pTemp, sizeof(DWORD), count, m_fp);				


				if (strcmp(szTagName, "Mesh")==0)
				{
					//MSG("** PHASE = %d\n", phase);
					// Read numVertexes
					if (phase == 0)
						pMesh->m_nNumVerts = pTemp[0];

					// Read face structure.
					if (phase == 2)
					{
						DWORD *p = pTemp;
						pMesh->m_nNumFaces = *p;	p++;
						count--;
						while (!count)
						{
							IXFile_Face *pFace = new IXFile_Face;
							int n = *p; p++;
							while (n)
							{
								pFace->indexes.push_back( (DWORD)*p);
								p++;
								n--;
							}
							pMesh->m_pFace.push_back(pFace);
						}
					}		

					phase++;
				}
				else if (strcmp(szTagName, "MeshNormals")==0)					
				{
					pMesh->m_nNumNormals = pTemp[0];
					pTemp = 0;
				}
				else if (strcmp(szTagName, "MeshTextureCoords")==0)					
				{
					pMesh->m_nUVVerts = pTemp[0];
					pTemp = 0;
				}
				else if (strcmp(szTagName, "MeshMaterialList")==0)					
				{
					// TODO : material ...
					printf("Read %d integers\n", count);
					pMesh->m_nNumMaterials = pTemp[0];
					pMesh->m_nNumMaterialId = pTemp[1];
					pMesh->m_pMaterialId = new DWORD[count-2];
					memcpy(pMesh->m_pMaterialId, pTemp + 2, count - 2);

				}
				else
				{
					printf("TAGi: %s\n", szTagName);
				}
				//MSG("%d integers\n", count);
				if (count == 1)
				{
					//MSG("v=%d\n", pTemp[0]);
				}
				delete[] pTemp;
			}
			break;

			case TOKEN_FLOAT_LIST: 
			{
				DWORD count;
				fread(&count, sizeof(DWORD), 1, m_fp);
				#ifdef _LITTLE_ENDIAN_
					#error "This code is not yet modified for little endianness!!"
				#endif
				float *pTemp = new float[count];
				ASSERT(pTemp);
				fread(pTemp, sizeof(float), count, m_fp);				
				//MSG("%d float\n", count);
				if (strcmp(szTagName, "Mesh")==0)
				{
					if (phase == 1)
					{
						pMesh->m_pVertex = (IXFile_Vertex*)pTemp;			
						pTemp = 0;
					}
					
					phase++;
				}
				else if (strcmp(szTagName, "MeshNormals")==0)					
				{
					pMesh->m_pNormals = (IXFile_Vertex*)pTemp;			
					pTemp = 0;
				}
				else if (strcmp(szTagName, "MeshTextureCoords")==0)					
				{
					pMesh->m_pUV = (IXFile_UV*)pTemp;
					pTemp = 0;
				}
				else if (strcmp(szTagName, "Material")==0)					
				{
					pMesh->m_pMaterial.push_back((IXFile_Material*) pTemp);
					pTemp = 0;
				}
				else
				{
					printf("TAGf: %s\n", szTagName);
				}
				
				if (pTemp)
					delete[] pTemp;				
				
			}
			break;
			
			default:
				//MSG("unknown token %d\n", idToken);
			break;

		}
	}

	//MSG("LAST MESH : %d faces, %d vertexes\n", pMesh->m_nNumFaces, pMesh->m_nNumVerts);
	if (pMesh)
		m_pMeshes.push_back(pMesh);
	return true;
}