#ifndef _IXFILEFORMAT_HH
#define _IXFILEFORMAT_HH


#include <iostream>
#include <list>

#ifdef _WIN32
using namespace std;
#endif


typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef float FLOAT;

struct COLORRGBAF {
	FLOAT r, g, b, a;
};

struct COLORRGBA {
	FLOAT r, g, b;
};

struct IXFile_Header {
	BYTE	m_szMagicNumber[4];
	WORD	m_iMajorVersion;
	WORD	m_iMinorVersion;
	BYTE	m_szFormatType[4];
	BYTE	m_szFloatSize[4];
};

struct IXFile_Vertex {
	FLOAT	x,y,z;
};

struct IXFile_UV {
	FLOAT	u,v;
};

struct IXFile_Face {
	list <DWORD>	indexes;
	DWORD			idMaterial;
};

struct IXFile_Material{
	COLORRGBAF		m_faceColor;
	FLOAT			m_power;
	COLORRGBA		m_Specular;
	COLORRGBA		m_Emissive;	
};

struct IXFile_Mesh {
	IXFile_Mesh() : m_nNumVerts(0), m_nNumFaces(0), m_nUVVerts(0), m_nNumNormals(0), m_nNumMaterials(0), m_nNumMaterialId(0)  { }
	char				m_szName[128];
	DWORD				m_nNumVerts;
	DWORD				m_nNumFaces;
	DWORD				m_nUVVerts;
	DWORD				m_nNumNormals;
	DWORD				m_nNumMaterials;
	DWORD				m_nNumMaterialId;	
	DWORD			*	m_pMaterialId;
	list<IXFile_Face*>	m_pFace;
	list<IXFile_Material*> m_pMaterial;
	IXFile_Vertex	*	m_pVertex;
	IXFile_Vertex	*	m_pNormals;
	IXFile_UV		*	m_pUV;
};

class IXFile
{
	public:
		IXFile(FILE *fp);
		IXFile(const char *szFilename);
		~IXFile();

		list<IXFile_Mesh*> m_pMeshes;

	private:
		bool ParseHeader();
		bool ParseFile();
		bool ParseTextFormat();
		bool ParseBinFormat();
		int GetWord(char *tmp, int iBufSize);

		IXFile_Header	m_Header;
		FILE		*	m_fp;
};

#endif // _IXFILEFORMAT_HH